//Look to LaggsDefault2_aiObjectives.cs, aiNeedRide.cs file for the root code. This file simply makes
//adjustments to the root code for mission specific ai behavior.


$aiVehicleWaitTime = 35000;
$aiNeedARideTaskTime = 200000;

function SiegeGame::onAIRespawn(%game, %client)
{
   //add the default tasks
   if (! %client.defaultTasksAdded)
   {
      %client.defaultTasksAdded = true;
      %client.addTask(AIEngageTask);
      %client.addTask(AIPickupItemTask);
      if (%client.team != Game.offenseTeam)
         %client.addTask(AITauntCorpseTask);

      %client.addTask(AIEngageTurretTask);
      %client.addtask(AIDetectMineTask);
      %client.addtask(AIDetectRemeqTask);//- Lagg... 3-20-2003
      %client.addtask(AIDetectVehicleTask);//- Lagg... 11-6-2003
      %client.addTask(AICouldUseInventoryTask);//- Lagg... 9-6-2003
   }
}

function AIOPilotShrike::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //check if flight path exists for team and game type
   if (Game.class $= "SiegeGame")
      %team = Game.offenseTeam == %client.team ? 1 : 2;
   else
      %team = %client.team;

   if(! isObject(nameToId("T" @ %team @ "ShrikePath1")))
      return 0;

   //do a quick check to disqualify this objective if it can't meet the minimum weight
   if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
   {
      if (%this.targetObjectId > 0 && %this.issuedByClientId == %client.controlByHuman)
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
	    return 0;
      }
      else
         return 0;
   }

   //check for Vehicle station near buy of forget it if we are not mounted yet - Lagg...
   if (!%client.player.isMounted())
   {
      //check how many vehicles of this type are in the field - Lagg..
      %blockName = "ScoutFlyer";
      if (!vehicleCheck(%blockName, %client.team))
         return 0;

      %clVs = AIFindClosestVStation(%client);
      if (%clVs > 0)
      {
         %closestVs = getWord(%clVs, 0);
         %closestVsDist = getWord(%clVs, 1);
         if (%closestVsDist > 300)//----------------------* close to VPad or return 0 *---
            return 0;
      }
      else
         return 0;
   }

   %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

   //never bump a shrike pilot from his ride - Lagg...
   if (%this.clientLevel1 > 0  && %level == 1 && %this.clientLevel1 != %client)
      return 0;
   else if (%this.clientLevel1 == %client)
      return 20000;

   // - add mission specific extras here - Lagg...
   if (nameToId("Team2TurretBaseLarge1").getDamageState() $= "Destroyed" &&
     nameToId("Team2TurretBaseLarge2").getDamageState() $= "Destroyed" &&
     nameToId("Team2TurretBaseLarge3").getDamageState() $= "Destroyed")
      return 0;

   return %weight;
}

//---------------------------------------------------------------------------------------------------------------------------

function AIONeedARide::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //check if flight path exists
   if (Game.class $= "SiegeGame")
      %team = Game.offenseTeam == %client.team ? 1 : 2;
   else
      %team = %client.team;

   if(! isObject(nameToId("T" @ %team @ "HAPCPath1")))
      return 0;

   //never bump a pilot off his ride
   if (%this.clientLevel1 > 0  && %level == 1 && %this.clientLevel1 != %client)
      return 0;
   else if (%this.clientLevel1 == %client && %level == 1)
      return 20000;

   //do a quick check to disqualify this objective if it can't meet the minimum weight
   if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
   {
      if (%this.targetObjectId > 0 && %this.issuedByClientId == %client.controlByHuman)
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
	    return 0;
      }
      else
         return 0;
   }

   //check for Vehicle station near buy of forget it if we are noy mounted yet - Lagg...
   if (!%client.player.isMounted())
   {
      %clVs = AIFindClosestVStation(%client);
      if (%clVs > 0)
      {
         %closestVs = getWord(%clVs, 0);
         %closestVsDist = getWord(%clVs, 1);
         if (%closestVsDist > 300)//----------------------* close to VPad or return 0 *---
            return 0;
      }
      else 
         return 0;

      //check how many vehicles of this type are in the field - Lagg..
      %blockName = "HAPCFlyer";
      if (!vehicleCheck(%blockName, %client.team))
         return 0;
   }

   // - add mission specific extras here - Lagg...

   //check for deployed MPB
   %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
   %closestMPB = getWord(%result, 0);
   //%closestMPBDist = getWord(%result, 1);

   //if the mpb is deployed and ent ff are down forget this task     
   if (%closestMPB > 0 && nameToId("EntranceFFGroup").powerCount <= 0)
      return 0;

   return AIODefault::weight(%this, %client, %level, %inventoryStr);
}

// -------------------------------------------------------------------------------------------

function AIONeedARide::assignClient(%this, %client)
{
   %client.objectiveTask = %client.addTask(AINeedARide);
   %client.objectiveTask.initFromObjective(%this, %client);

   //create the escort objectives

   //pick a random equipment set...
   %randNum = getRandom();
   if (%randNum < 0.2)
   {
      %desEquip = "PlasmaPlasmaAmmo";
      %set = "HeavyEnergySet";
   }
   else if (%randNum < 0.4)
   {
      %desEquip = "InventoryDeployable";
      %set = "MediumInventorySet";
   } 
   else if (%randNum < 0.6)
   {
      %desEquip = "AmmoPack";
      %set = "HeavyAmmoSet";
   }
   else if (%randNum < 0.8)
   {
      %desEquip = "ShieldPack";
      %set = "HeavyShieldOff";
   }
   else
   {
      %desEquip = "AmmoPack";
      %set = "MediumMissileSet";
   }

   %client.escort = new AIObjective(AIOEscortPlayer)
                     {
						      dataBlock = "AIObjectiveMarker";
                        weightLevel1 = 5000;//set high - lagg...
                        weightLevel2 = 0;
                        description = "Escort " @ getTaggedString(%client.name);
                        targetClientId = %client;
                        offense = true;
                        chat = "ChatNeedHold";
                        desiredEquipment = "FlareGrenade";
								buyEquipmentSet = "HeavyRepairSet";
                     };

   %client.escort1 = new AIObjective(AIOEscortPlayer)
                     {
						      dataBlock = "AIObjectiveMarker";
                        weightLevel1 = 5000;
                        weightLevel2 = 0;
                        description = "Escort " @ getTaggedString(%client.name);
                        targetClientId = %client;
                        offense = true;
                        chat = "ChatNeedRide";
                        desiredEquipment = %desEquip;
								buyEquipmentSet = %set;
                     };

   //pick a random equipment set...
   %randNum = getRandom();
   if (%randNum < 0.3)
   {
      %desEquip = "InventoryDeployable";
      %set = "HeavyInventorySet";
   }
   else if (%randNum < 0.5)
   {
      %desEquip = "PlasmaPlasmaAmmo";
      %set = "MediumShieldSet";
   } 
   else if (%randNum < 0.6)
   {
      %desEquip = "SensorJammerPack";
      %set = "HeavyEnergySet";
   }
   else if (%randNum < 0.8)
   {
      %desEquip = "ShieldPack";
      %set = "HeavyShieldOff";
   }
   else
   {
      %desEquip = "AmmoPack";
      %set = "MediumMissileSet";
   }

   %client.escort2 = new AIObjective(AIOEscortPlayer)
                     {
						      dataBlock = "AIObjectiveMarker";
                        weightLevel1 = 5000;
                        weightLevel2 = 0;
                        description = "Escort " @ getTaggedString(%client.name);
                        targetClientId = %client;
                        offense = true;
                        chat = "ChatNeedRide";
                        desiredEquipment = %desEquip;
								buyEquipmentSet = %set;
                     };

   MissionCleanup.add(%client.escort);
   MissionCleanup.add(%client.escort1);
   MissionCleanup.add(%client.escort2);

   //if is siege game we have to do this right - Lagg... 11-3-2003
   if (Game.class $= "SiegeGame")
   {
      //the objectives in $ObjectiveQ[1] are all offense objectives, $ObjectiveQ[2] defense
      if (%client.team == game.offenseTeam)
      {
         $ObjectiveQ[1].add(%client.escort);
         $ObjectiveQ[1].add(%client.escort1);
         $ObjectiveQ[1].add(%client.escort2);
      }
      else
      {
         $ObjectiveQ[2].add(%client.escort);
         $ObjectiveQ[2].add(%client.escort1);
         $ObjectiveQ[2].add(%client.escort2);
      }
   }
   else
   {
         $ObjectiveQ[%client.team].add(%client.escort);
         $ObjectiveQ[%client.team].add(%client.escort1);
         $ObjectiveQ[%client.team].add(%client.escort2);
   }
}

//--------------------------------------------------------------------------------------------------- AIO Attack Location ---

function AIOAttackLocation::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;
      
   //now, if this bot is linked to a human who has issued this command, up the weight
   if (%this.issuedByClientId == %client.controlByHuman)
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
	    return 0;
	 else
	    %weight = $AIWeightHumanIssuedCommand;
      }
      else
      {
         // calculate the default...
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	 if (%weight < $AIWeightHumanIssuedCommand)
	    %weight = $AIWeightHumanIssuedCommand;
      }
   }
   else
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
         return 0;

      // - add mission specific extras here - Lagg...
      if (%this.mode $= "EntFFUp")
      {
         if (nameToId("EntranceFFGroup").powerCount <= 0 || %client.inBase)
            return 0;
      }

      // calculate the default...
      %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
   }

   return %weight;
}

//----------------------------------------------------------------------------------------------- AIO Repair Object ---

function AIORepairObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team != %client.team)
      return 0;      
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //check to see if we are in base or a paticular gen is up/down - mission specific here - Lagg...
   if (%this.mode $= "inField")
   {
      if (%client.inBase)
         return 0;
   }
   else if (%this.mode $= "EntFFUp")//this one is for the defense team
   {
      if (nameToId("EntranceFFGroup").powerCount < 1)
         return 0;
   }
   else if (%this.mode $= "EntFFDown")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 || %client.inBase)
         return 0;
   }
   else if (%this.mode $= "GenAtt")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
         return 0;
   }
   else if (%this.mode $= "mainGensDown")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
         return 0;

      if(nameToId("MainPower").powerCount > 0)
         return 0;
   }
      
   //no need to repair if the object isn't in need
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamagePercent() <= 0)
      return 0;
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
      if (%this.issuedByClientId == %client.controlByHuman)
      {
         //make sure we have the potential to reach the minWeight
         if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
         {
            if ($AIWeightHumanIssuedCommand < %minWeight)
               return 0;
	    else
	       %weight = $AIWeightHumanIssuedCommand;
         }
         else
	 {
	    // calculate the default...
	    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	    if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
         }
      }
      else
      {
         //make sure we have the potential to reach the minWeight
         if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
            return 0;

         // calculate the default...
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
      }

      return %weight;
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Escort Player +++
//modified escorter armor type allowed, rewritten all aivehicle escort stuff, included vehicleobjecttypes
//as potential targets if lazed by escortie.                                              - Lagg... - 4-12-2003

function AIOEscortPlayer::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client) || ! AIClientIsAlive(%this.targetClientId))
      return 0;

   //can't escort yourself
   if (%client == %this.targetClientId)
      return 0;

   //if vehicle mounted escort check for empty seats - Lagg... - 5-17-2003
   %targMounted = %this.targetClientId.player.ismounted();
   if (%targMounted > 0)
   {
      //%this.forceClientId = %client;//this should stop em getting bumped
      %vehicle = %this.targetClientId.vehicleMounted;
      %node = findAIEmptySeat(%vehicle, %client.player);

      if (%node < 0 && %client.vehicleMounted != %this.targetClientId.vehicleMounted)
         return 0;

      // - add mission specific zone checks here - Lagg... 
      if (%client.inBase)
         return 0;
   }

   //make sure the class is appropriate unless we are in a vehicle :) - Lagg...
   if (%this.forceClientId <= 0 && %this.issuedByClientId != %client.controlByHuman)
   {
      %targArmor = %this.targetClientId.player.getArmorSize();
      %myArmor = %client.player.getArmorSize();
      %targMounted = %this.targetClientId.player.ismounted();
                      
      //if ((%targArmor $= "Light" && %myArmor !$= "Light" && !%targMounted)
        //|| (%targArmor $= "Medium" && %myArmor $= "Heavy" && !%targMounted))
	      //return 0;

      //changed here to allow bigger escorts - Lagg... 1-31-2003
      if (%targArmor $= "Light" && %myArmor $= "Heavy" && !%targMounted)
	      return 0;
   }

   //can't bump a forced client from level 1
   if (%this.forceClientId > 0 && %this.forceClientId != %client && %level == 1)
      return 0;

   //lets check the range on this one if not forced from command map
   if (%this.forceClientId <= 0 && vectorDist(%client.Player.getWorldBoxCenter(), %this.targetClientId.player.getWorldBoxCenter()) > 200)
      return 0;

   //if this bot is linked to a human who has issued this command, up the weight
   if (%this.issuedByClientId == %client.controlByHuman)
   {
      //make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
		{
			if ($AIWeightHumanIssuedEscort < %minWeight)
				return 0;
			else
				%weight = $AIWeightHumanIssuedEscort;
		}
		else
		{
			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
			if (%weight < $AIWeightHumanIssuedEscort)
				%weight = $AIWeightHumanIssuedEscort;
		}
	}
	else
	{
		//make sure we have the potential to reach the minWeight
		if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			return 0;

		// calculate the default...
		%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	}

	return %weight;
}

// --------------------------------------------------------------------------------------------------------------------------------- AIO MDestroy Object --- OK *

function AIOMDestroyObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;      
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   // - add mission specific zone checks here - Lagg... 
   if (%this.mode $= "EntFFDown")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
         return 0;
   }
      
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
	{
		//if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman)
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			{
				if ($AIWeightHumanIssuedCommand < %minWeight)
					return 0;
				else
					%weight = $AIWeightHumanIssuedCommand;
			}
			else
			{
				// calculate the default...
				%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
				if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
			}
		}
		else
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
				return 0;

			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
		}

      //if we are playing DnD, check if objective was completed already, if not give some extra :) - Lagg... 3-29-2004
      if (Game.class $= "DnDGame")
      {
         if (%this.targetObjectId.objectiveCompleted)
            %weight -= 400;
         else
            %weight += 200;
      }

      return %weight;
   }
}

// --------------------------------------------------------------------------------------------------------------------------------- AIO Defend Location ---

function AIODefendLocation::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (%this.targetObjectId > 0  && %this.defense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team != %client.team)
         return 0;
   }
   else if (%this.targetObjectId > 0  && %this.offense > 0)
   {
      if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
         return 0;

      //don't camp unless base is down
      if (%this.targetObjectId.getGroup().powerCount > 0)
         return 0;
   }
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

	//do a quick check to disqualify this objective if it can't meet the minimum weight
	if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	{
		if (%this.targetObjectId > 0 && %this.issuedByClientId == %client.controlByHuman)
		{
			if ($AIWeightHumanIssuedCommand < %minWeight)
				return 0;
		}
		else
			return 0;
	}

        // - add mission specific zone checks here - Lagg... 
        if (%this.mode $= "EntFFDown")
        {
           if (%client.inBase)
              return 0;
           if (nameToId("ObjectiveGroup").powerCount == 0)
              return 0;
        }
        else if (%this.mode $= "ResGenDown")
        {
           if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
              return 0;
           if (nameToId("ObjectiveGroup").powerCount > 0)
              return 0;
        }
        else if (%this.mode $= "EntFFUp")
        {
           if (nameToId("EntranceFFGroup").powerCount <= 0 || %client.inBase)
              return 0;
        }
        else if (%this.mode $= "Objective")
        {
           if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
              return 0;
           if (nameToId("ObjectiveGroup").powerCount > 0)
              return 0;
        }

	%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

        //if defensive  the object has been destroyed, reduce the weight / if offensive increase the weight if destroyed (Camping !)
	if (%this.targetObjectId > 0 && %this.defense > 0)
	{

		//see if we were forced on the objective
		if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
			%weight = $AIWeightHumanIssuedCommand;

		//else see if the object has been destroyed - slight modification here - Lagg... 11-7-2003
	   //else if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
           else if (!isObject(%this.targetObjectId) || %this.targetObjectId.isDisabled())
			%weight -= 320;
	}
        else if (%this.targetObjectId > 0 && %this.offense > 0)
	{

		//see if we were forced on the objective
		if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
			%weight = $AIWeightHumanIssuedCommand;

		// - slight modification here - Lagg... 11-7-2003
                else if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() !$= "Destroyed")
			return 0;//don't camp if not destroyed
                if (%client.player.getInventory("InventoryDeployable") > 0)
                   return 0;
	}


	return %weight;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Destroy Object +++
//place objective marker in spot you want bot to stand and destroy for sentry turrets and hard to
//reach spots. basically same as attackobject, use this on generators/solar panels in siege game - Lagg... 3-26-2003

function AIODestroyObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;      
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;
      
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
      if (%this.issuedByClientId == %client.controlByHuman)
      {
         //make sure we have the potential to reach the minWeight
         if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	 {
	    if ($AIWeightHumanIssuedCommand < %minWeight)
	       return 0;
	    else
	       %weight = $AIWeightHumanIssuedCommand;
	 }
	 else
	 {
	    // calculate the default...
	    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	    if (%weight < $AIWeightHumanIssuedCommand)
	       %weight = $AIWeightHumanIssuedCommand;
	 }
      }
      else
      {
         //make sure we have the potential to reach the minWeight
         if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
            return 0;

         // calculate the default...
         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         // - add mission specific zone checks here - Lagg... 
         %name = %this.targetObjectId.getName();

         if (%client.inBase && (%name $= "Team2StationInventory1" || %name $= "Team2StationInventory2" || %name $= "Team2StationInventory3" ||
           %name $= "Team2StationInventory4" || %name $= "Team2StationInventory5" || %name $= "Team2StationInventory6"))
            return 0;
         else if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase &&
           %name !$= "Team2StationInventory1" && %name !$= "Team2StationInventory2" &&
           %name !$= "Team2StationInventory3" && %name !$= "Team2StationInventory4" &&
           %name !$= "Team2StationInventory5" && %name !$= "Team2StationInventory6")
            return 0;
      } 

      //lets give him some personallity on this one :) - Lagg...
      if ((findTargetInView(%client, 150)) == %this.targetObjectId)
         %weight += 400;

      return %weight;
   }
}

//------------------------------------------------------------------------------------------------------ AIO Attack Object ---
//totally rewritten so but will RAPE the object. used only for generators in siege
//also bots will follow a random path to target if not already in the base - Lagg... 11-1-2003

function AIOAttackObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;      
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;               
                 
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
      if (%this.issuedByClientId == %client.controlByHuman)
      {
         //make sure we have the potential to reach the minWeight
	 if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	 {
	    if ($AIWeightHumanIssuedCommand < %minWeight)
	       return 0;
	    else
	       %weight = $AIWeightHumanIssuedCommand;
         }
	 else
	 {
	    // calculate the default...
	    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	    if (%weight < $AIWeightHumanIssuedCommand)
	       %weight = $AIWeightHumanIssuedCommand;
	 }
      }
      else
      {
         //make sure we have the potential to reach the minWeight
	 if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	    return 0;

         //------------------------------------------------------- if target == EntFFGenerator1/2 ---
         if (%this.targetObjectId == nameToId("Team2generatorLarge1") || %this.targetObjectId == nameToId("Team2generatorLarge2"))
         {
            if (%client.inBase)
               return 0;

            if (nameToId("Team2TurretBaseLarge1").getDamageState() $= "Destroyed")
            {
               //this should break any defense around the Entrance FF gens
               %this.weightLevel1 = "4100";
               %this.weightLevel2 = "4100";
               %this.weightLevel3 = "3400";
               %this.weightLevel4 = "3400";
            }
            else
            {
               //set weight normal
               %this.weightLevel1 = "3400";
	       %this.weightLevel2 = "1900";
	       %this.weightLevel3 = "0";
	       %this.weightLevel4 = "0";
            }   
         }

         // calculate the default...
	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         // ---------------------------------------------------------------- Zone Checks ---
         %name = %this.targetObjectId.getName();

         if (nameToId("EntranceFFGroup").powerCount > 0 && %name !$= "Team2generatorLarge1" && %name !$= "Team2generatorLarge2" && !%client.inBase)
            return 0;
      }

      //lets give him some personallity on this one :) - Lagg...
      if ((findTargetInView(%client, 250)) == %this.targetObjectId)
         %weight += 200;

      return %weight;
   }
}

//---------------------------------------------------------------------------------------------- AI Attack Object ---

function AIAttackObject::initFromObjective(%task, %objective, %client)
{
   %task.baseWeight = %client.objectiveWeight;
   %task.targetObject = %objective.targetObjectId;
   %task.equipment = %objective.equipment;
   %task.buyEquipmentSet = %objective.buyEquipmentSet;
   %task.desiredEquipment = %objective.desiredEquipment;
   %task.issuedByClient = %objective.issuedByClientId;
   %task.location = %objective.location;

   //follow path - Lagg...
   %task.followPath = false;

   //initialize other task vars
   %task.sendMsg = true;
   %task.sendMsgTime = 0;
   %task.useTeleport = false;
}

function AIAttackObject::assume(%task, %client)
{
   %task.setWeightFreq(15);
   %task.setMonitorFreq(15);

   //added here so Bot buys a random set if no set was specified in objective dataBlock - Lagg... 2-14-2003
      if (%task.buyEquipmentSet $= "randomSet")
      {
         //pick a random equipment set...
         %randNum = getRandom();
         if (%randNum < 0.2)
         {
            %task.desiredEquipment = "Plasma PlasmaAmmo";
            %task.buyEquipmentSet = "HeavyEnergySet";
         }
         else if (%randNum < 0.4)
         {
            %task.desiredEquipment = "Plasma PlasmaAmmo";
            %task.buyEquipmentSet = "MediumEnergySet";
         } 
         else if (%randNum < 0.6)
         {
            %task.desiredEquipment = "Plasma PlasmaAmmo";
            %task.buyEquipmentSet = "LightCloakSet";
         }
         else if (%randNum < 0.8)
         {
            %task.desiredEquipment = "Plasma PlasmaAmmo";
            %task.buyEquipmentSet = "HeavyShieldOff";
         }
         else
         {
            %task.desiredEquipment = "Plasma PlasmaAmmo";
            %task.buyEquipmentSet = "MediumMissileSet";
         }
      }

   if (%task.equipment !$= "")
      %equipmentList = %task.equipment;
   else
      %equipmentList = %task.desiredEquipment;
   %client.needEquipment = AINeedEquipment(%equipmentList, %client);

   //even if we don't *need* equipemnt, see if we should buy some... 
   if (!%client.needEquipment && %task.buyEquipmentSet !$= "")
   {
      //see if we could benefit from inventory
      %needArmor = AIMustUseRegularInvStation(%task.desiredEquipment, %client);
      %result = AIFindClosestInventory(%client, %needArmor);
      %closestInv = getWord(%result, 0);
      %closestDist = getWord(%result, 1);
      if (AINeedEquipment(%task.desiredEquipment, %client) && %closestInv > 0)
      {
         //find where we are
         %clientPos = %client.player.getWorldBoxCenter();
         %objPos = %task.targetObject.getWorldBoxCenter();
         %distToObject = %client.getPathDistance(%objPos);

	 if (%distToObject < 0 || %closestDist < 25)//-------- 25m or less to inventoryStation in this case
            %client.needEquipment = true;
      }
   }

   //mark the current time for the buy inventory state machine
   %task.buyInvTime = getSimTime();

   //--------------------------------------------------- follow paths ---
   //set the destination paths for each team and game type
   
   //first see how many paths we have
   //check if the path exists
   if (Game.class $= "SiegeGame")
      %team = Game.offenseTeam == %client.team ? 1 : 2;
   else
      %team = %client.team;

   %mx = 0;
   for(%i = 1; %i < 10; %i++)
   {
      if(isObject(nameToId("T" @ %team @ "FollowPath" @ %i)))
         %mx = %i;
   }

   %name = %task.targetObject.getName();
   if (%mx > 0 && %name !$= "Team2generatorLarge1" && %name !$= "Team2generatorLarge2" && !%client.inBase)
   {
      %random = mFloor(getRandom(1, %mx));

      // - set the path simgroup to follow
      %task.group = nameToId("T" @ %team @ "FollowPath" @ %random);
      %task.count = %task.group.getCount();  
      %task.locationIndex = 0;
      %task.followPath = true;
   }

   //--------------------------------------------------- assume telporter - start -
   //if the MPB Teleporter is online - Lagg... 9-30-2003

   //first check for deployed MPB
   %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
   %closestMPB = getWord(%result, 0);
   %closestMPBDist = getWord(%result, 1);
      
   //next find the teleporter
   %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
   %closestTel = getWord(%result, 0);
   %closestTelDist = getWord(%result, 1);
      
   //now check for what would be closer - Lagg... 10-1-2003
   if (%closestMPB > 0 && %closestTel > 0)
   {
      %disToTarg = %client.getPathDistance(%task.targetObject.getTransform());
      %mpbDisToTarg = vectorDist(%closestMPB.getTransform(), %task.targetObject.getTransform());
         
      if (%closestTelDist < %closestMPBDist && %closestMPBDist < %disToTarg && %mpbDisToTarg < %disToTarg)
         %task.useTeleport = true;//new - Lagg... 10-1-2003
      else
         %task.useTeleport = false;             
   }
   //--------------------------------------------------- assume telporter - end -
}

function AIAttackObject::retire(%task, %client)
{
   %client.setTargetObject(-1);
}

function AIAttackObject::weight(%task, %client)
{
   //update the task weight
   if (%task == %client.objectiveTask)
      %task.baseWeight = %client.objectiveWeight;

   //see if we can find someone to shoot at...
   if (%client.getEngageTarget() <= 0)
   {
      %losTimeout = $AIClientMinLOSTime + ($AIClientLOSTimeout * %client.getSkillLevel());
      %myLocation = %client.player.getWorldBoxCenter();
      %result = AIFindClosestEnemy(%client, 40, %losTimeout);
      %task.engageTarget = getWord(%result, 0);
   }
   
   %task.setWeight(%task.baseWeight);
}

function AIAttackObject::monitor(%task, %client)
{
   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25, $AIModeWalk);
            return;
         }
      }
      else
         %task.useTeleport = false;
   }
   //--------------------------------------------------- monitor telporter - end -

   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   %hasMortar = (%client.player.getInventory("Mortar") > 0) && (%client.player.getInventory("MortarAmmo") > 0);
   %hasPlasma = (%client.player.getInventory("Plasma") > 0) && (%client.player.getInventory("PlasmaAmmo") > 0);
   %hasDisc = (%client.player.getInventory("Disc") > 0) && (%client.player.getInventory("DiscAmmo") > 0) && (!%client.player.getInventory("Blaster") > 0);
   if (%hasMortar || %hasPlasma || %hasDisc)
      %client.needEquipment = false;

   //first, buy the equipment - if we really really need some :) - Lagg... - 4-6-2003
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
         %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         //if this task is the objective task, choose a new objective
	 if (%task == %client.objectiveTask)
	 {
	    AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	 }
         return;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();
   
   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
         {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
            }
	    else if (%task.targetObject > 0)
	    {
	       %type = %task.targetObject.getDataBlock().getName();
	       if (%type $= "GeneratorLarge")
	          AIMessageThreadTemplate("AttackBase", "ChatSelfAttackGenerator", %client, -1);
	       else if (%type $= "SensorLargePulse")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
	       else if (%type $= "SensorMediumPulse")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackSensors", %client, -1);
	       else if (%type $= "TurretBaseLarge")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackTurrets", %client, -1);
	       else if (%type $= "StationVehicle")
		  AIMessageThreadTemplate("AttackBase", "ChatSelfAttackVehicle", %client, -1);
            }
         }
      }
   }

   //set the target object if targetObject not destroyed
   %outOfStuff = (AIAttackOutofAmmo(%client));
   if (isObject(%task.targetObject) && %task.targetObject.getDamageState() !$= "Destroyed" && !%outOfStuff)
   {
      if (%task.followPath)
      {
         //set the path and follow it to the end
         %task.spot = %task.group.getObject(%task.locationIndex).position;

         %pos = %client.player.position;
         %dest = %task.spot;

         //are we close to location index marker?
         if (VectorDist(%dest, %pos) < 6)//6 meters from marker
         {
            //if we have another location index
            if ((%task.count - 1) > %task.locationIndex)//has to be -1 on the groups count
               %task.locationIndex++;
            //else we are at end of trail
            else
            {
               //if this task is the objective task, choose a new objective
	       if (%task == %client.objectiveTask)
	       {
	          %task.followPath = false;
	          return;
               }
            }
         }
         else
            %client.stepMove(%task.spot, 0.25, $AIModeExpress);
      }
      else
         %client.stepMove(%task.location, 0.25, $AIModeExpress);//move to target no matter what

      %targetLOS = "false";
      %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;
      %targetLOS = !containerRayCast(%client.player.getMuzzlePoint($WeaponSlot), %task.targetObject.getWorldBoxCenter(), %mask, 0);

      %close = false;
      if (VectorDist(%task.targetObject.position, %client.player.position) < 5)
         %close = true;
      
      //fire at target object if in sight
      if (%targetLOS && !%close)
      {
         %task.setWeight($AIWeightReturnFire);
         %client.setEngageTarget(-1);
         %client.setTargetObject(%task.targetObject, 100, "Mortar");
      }
      else
      {
         %client.setTargetObject(-1);
         %task.setWeight(%task.baseWeight);

         //see if we need to engage a new target
         %engageTarget = %client.getEngageTarget();
         if (!AIClientIsAlive(%engageTarget) && %task.engageTarget > 0)
            %client.setEngageTarget(%task.engageTarget);

         //else see if we should abandon the engagement
         else if (AIClientIsAlive(%engageTarget))
         {
            %myPos = %client.player.getWorldBoxCenter();
            %testPos = %engageTarget.player.getWorldBoxCenter();
            %distance = %client.getPathDistance(%testPos);
            if (%distance < 0 || %distance > 20)
               %client.setEngageTarget(-1);

            //see if we're supposed to be engaging anyone...
            if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
               %client.setEngageTarget(%client.shouldEngage);
         }
      } 
   }
   else
   {
      %client.setTargetObject(-1);      
            
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         //if we need to repair self, aim at the floor
         if (%client.player.getDamagePercent() > 0 && %client.player.getInventory(RepairPack) > 0)
         {
            %pos = %client.player.getWorldBoxCenter();
            %posX = getWord(%pos, 0);
            %posY = getWord(%pos, 1);
            %posZ = getWord(%pos, 2) - 1;
            %pos = %posX SPC %posY SPC %posZ;
            %client.aimAt(%pos, 500);
         }
         AIUnassignClient(%client);
         Game.AIChooseGameObjective(%client);
      }
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Deploy Equipment +++
//modified here so bot will continue deploying if has equip and not get bumped
//can be set at a realistic weight now, like 3400               - Lagg... 4-12-2003
//If the function ShapeBaseImageData::testInvalidDeployConditions() changes at all, those changes need to be reflected here

function AIODeployEquipment::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //make sure the deploy objective is valid
   if (%this.isInvalid)
      return 0;

   //if badly damaged and we have a repair pack, don't waste time
   %damage = %client.player.getDamagePercent();
   if (%client.player.getInventory("RepairPack") > 0 && %damage > 0.3)
      return 0;


   //special checks to see if we are in base or a paticular gen is up/down
   if (%this.mode $= "inField")
   {
      if (%client.inBase)
         return 0;
   }
   else if (%this.mode $= "EntFFUp")//this one is for the defense team
   {
      if (nameToId("EntranceFFGroup").powerCount < 1)
         return 0;
   }
   else if (%this.mode $= "EntFFDown")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 || %client.inBase)
         return 0;
   }
   else if (%this.mode $= "MainGensDown")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
         return 0;

      if (nameToId("MainPower").powerCount > 0)
         return 0;

       //check equipment requirement - if we are in base we can't get armor
      %needEquipment = AINeedEquipment(%this.equipment, %client);
      %needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
      if (%client.inbase && %needEquipment && %needArmor)
         return 0;
   }
   else if (%this.mode $= "GenAtt")
   {
      if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
         return 0;

       //check equipment requirement - if we are in base we can get armor
      %needEquipment = AINeedEquipment(%this.equipment, %client);
      %needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
      if (%client.inbase && %needEquipment && %needArmor)
         return 0;
   }

   //first, make sure we haven't deployed too many...
   if (%this.equipment $= "TurretOutdoorDeployable" || %this.equipment $= "TurretIndoorDeployable")
      %maxAllowed = countTurretsAllowed(%this.equipment);
   else
      %maxAllowed = $TeamDeployableMax[%this.equipment];

   if ($TeamDeployedCount[%client.team, %this.equipment] >= %maxAllowed)
      return 0;

   //now make sure there are no other items in the way...
   InitContainerRadiusSearch(%this.location, $MinDeployableDistance, $TypeMasks::VehicleObjectType |
  	                                             $TypeMasks::MoveableObjectType |
  	                                             $TypeMasks::StaticShapeObjectType |
  	                                             $TypeMasks::TSStaticShapeObjectType | 
  	                                             $TypeMasks::ForceFieldObjectType |
  	                                             $TypeMasks::ItemObjectType | 
  	                                             $TypeMasks::PlayerObjectType | 
  	                                             $TypeMasks::TurretObjectType);
      %objSearch = containerSearchNext();

      //make sure we're not invalidating the deploy location with the client's own player object
      if (%objSearch == %client.player)
         %objSearch = containerSearchNext();

      //did we find an object which would block deploying the equipment?
      if (isObject(%objSearch))
         return 0;

   //now run individual checks based on the equipment type...
   if (%this.equipment $= "TurretIndoorDeployable")
   {
      //check if there's another turret close to the deploy location
      InitContainerRadiusSearch(%this.location, $TurretIndoorSpaceRadius, $TypeMasks::StaticShapeObjectType);
      %found = containerSearchNext();
      if (isObject(%found))
      {
         %foundName = %found.getDataBlock().getName();
 	 if ((%foundName $= TurretDeployedFloorIndoor) || (%foundName $= "TurretDeployedWallIndoor") ||
           (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 	    return 0;
      }
 
      //now see if there are too many turrets in the area...
      %highestDensity = 0;
      InitContainerRadiusSearch(%this.location, $TurretIndoorSphereRadius, $TypeMasks::StaticShapeObjectType);
      %found = containerSearchNext();
      while (isObject(%found))
      {
         %foundName = %found.getDataBlock().getName();
 	 if ((%foundName $= "TurretDeployedFloorIndoor") || (%foundName $= "TurretDeployedWallIndoor") ||
           (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 	 {
 	    //found one
 	    %numTurretsNearby++;//+++++++++++++++++++++++++++++++++++++++++ ahh review this - Lagg... +++
            %nearbyDensity = testNearbyDensity(%found, $TurretIndoorSphereRadius);
 	    if (%nearbyDensity > %highestDensity)
 	       %highestDensity = %nearbyDensity;
         }
 	 %found = containerSearchNext();
      }
 
      if (%numTurretsNearby > %highestDensity)
         %highestDensity = %numTurretsNearby;
 
      //now see if the area is already saturated
      if (%highestDensity > $TurretIndoorMaxPerSphere)
         return 0;
   }
   else if (%this.equipment $= "TurretOutdoorDeployable")
   {
      //check if there's another turret close to the deploy location
      InitContainerRadiusSearch(%this.location, $TurretOutdoorSpaceRadius, $TypeMasks::StaticShapeObjectType);
      %found = containerSearchNext();
      if (isObject(%found))
      {
         %foundName = %found.getDataBlock().getName();     
         if ((%foundName $= "TurretDeployedFloorIndoor") || (%foundName $= "TurretDeployedWallIndoor") || (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
            return 0;
      }
 
      //now see if there are too many turrets in the area...
      %highestDensity = 0;
      InitContainerRadiusSearch(%this.location, $TurretOutdoorSphereRadius, $TypeMasks::StaticShapeObjectType);
      %found = containerSearchNext();
      while (isObject(%found))
      {
         %foundName = %found.getDataBlock().getName();
 	 if ((%foundName $= "TurretDeployedFloorIndoor") || (%foundName $= "TurretDeployedWallIndoor") || (%foundName $= "TurretDeployedCeilingIndoor") || (%foundName $= "TurretDeployedOutdoor"))
 	 {
 	    //found one
 	    %numTurretsNearby++;
 
 	    %nearbyDensity = testNearbyDensity(%found, $TurretOutdoorSphereRadius);
 	    if (%nearbyDensity > %highestDensity)
 	       %highestDensity = %nearbyDensity;     
 	 }
 	 %found = containerSearchNext();
      }
 
      if (%numTurretsNearby > %highestDensity)
         %highestDensity = %numTurretsNearby;
 
      //now see if the area is already saturated
      if (%highestDensity > $TurretOutdoorMaxPerSphere)
         return 0;
   }

   //check equipment requirement
   %needEquipment = AINeedEquipment(%this.equipment, %client);
   %needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
   %result = AIFindClosestInventory(%client, %needArmor);
   %closestInv = getWord(%result, 0);
   %closestDist = getWord(%result, 1);
   if (%client.team == Game.offenseTeam && %needEquipment && %closestDist > 200)
   {
      %itemType = getWord(%this.requiredEquipment, 0);
      %found = false;
      %itemCount = $AIItemSet.getCount();
      for (%i = 0; %i < %itemCount; %i++)
      {
         %item = $AIItemSet.getObject(%i);
  	 if (%item.getDataBlock().getName() $= %itemType && !%item.isHidden())
  	 {
	    %weightDist = %client.getPathDistance(%item.getWorldBoxCenter());
	    if (%weightDist > 0 && %weightDist < 200)
	    {
	       //%weightInv = %item;  //set the var so the distance function will work...
	       %found = true;
	       break;
	    }
         }
      }
      if (!%found)
         return 0; 
   }
	
   //if we don't need equipment, see if we've past the "point of no return", and should continue regardless
   if (! %needEquipment && %this.clientLevel1 == %client)
   {
      //if we're too far from the inv to go back, or we're too close to the deploy location, force continue
      //if (%closestDist > 50 && VectorDist(%client.player.getWorldBoxCenter(), %task.location) < 50) - modified Lagg...
      if (%closestDist > 25)
      {
         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
 	 if (%weight < $AIWeightContinueDeploying)
 	    %weight = $AIWeightContinueDeploying;

         return %weight;
      }
   }

   //if this bot is linked to a human who has issued this command, up the weight
   if (%this.issuedByClientId == %client.controlByHuman)
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      {
         if ($AIWeightHumanIssuedCommand < %minWeight)
            return 0;
	 else
            %weight = $AIWeightHumanIssuedCommand;
      }
      else
      {
         // calculate the default...
         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	 if (%weight < $AIWeightHumanIssuedCommand)
	    %weight = $AIWeightHumanIssuedCommand;
      }
   }
   else
   {
      //make sure we have the potential to reach the minWeight
      if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
         return 0;

      // calculate the default...
      %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
   }
   //check equipment requirement
   //%needEquipment = AINeedEquipment(%this.equipment, %client);
   //%needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
   //%result = AIFindClosestInventory(%client, %needArmor);
   //%closestInv = getWord(%result, 0);
   //%closestDist = getWord(%result, 1);
   if (%needEquipment && !%needArmor && %closestDist > 200)
   {
      %found = false;
      %itemType = getWord(%this.requiredEquipment, 0);
      %itemCount = $AIItemSet.getCount();
      for (%i = 0; %i < %itemCount; %i++)
      {
         %item = $AIItemSet.getObject(%i);
  	 if (%item.getDataBlock().getName() $= %itemType && !%item.isHidden())
  	 {
	    %weightDist = %client.getPathDistance(%item.getWorldBoxCenter());
	    if (%weightDist > 0 && %weightDist < 200)
	    {
	       //%weightInv = %item;  //set the var so the distance function will work...
	       %found = true;
	       break;
	    }
         }
      }
      if (%found)
         %weight = $AIWeightContinueDeploying + 100;
      else
      // calculate the default...
         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr); 
   }
   else
      // calculate the default...
      %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

	return %weight;
}

//----------------------------------------------------------------------------------------------- AI Deploy Euipment ---

function AIDeployEquipment::monitor(%task, %client)
{
   //first, make sure we haven't deployed too many...
   if (%task.equipment $= "TurretOutdoorDeployable" || %task.equipment $= "TurretIndoorDeployable")
      %maxAllowed = countTurretsAllowed(%task.equipment);
   else
      %maxAllowed = $TeamDeployableMax[%task.equipment];

   if ($TeamDeployedCount[%client.team, %task.equipment] >= %maxAllowed)
   {
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
         return;
      }
   }

   //make sure this was not deployed already
   InitContainerRadiusSearch(%task.location, $MinDeployableDistance, $TypeMasks::StaticShapeObjectType);
   %found = containerSearchNext();
   if (isObject(%found))
   {
      if (%found.team == %client.team)
      {
         if (%task == %client.objectiveTask)
	 {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
	 }
      }
   }

   //see if we are hung up on a force field
   InitContainerRadiusSearch(%client.player.getWorldBoxCenter(), 4, $TypeMasks::ForceFieldObjectType);
   %objSearch = containerSearchNext();
   if (%objSearch > 0)
   {
      %ffDis = %client.getPathDistance(%objSearch.getWorldBoxCenter());
      %myDis = %client.getPathDistance(%task.position);
      if (%ffDis < %myDis )
      {  
         if (%objSearch.isPowered() && %objSearch.team != %client.team)
         {
            //if this task is the objective task, choose a new objective
	    if (%task == %client.objectiveTask)
	    {
               %client.setDangerLocation(%objSearch.getWorldBoxCenter(), 20);
	       AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
	    }
            return;
         }
      }
   }

   //first, see if we still need the deployable pack - Lagg... 2-25-2004
   %itemType = getWord(%task.equipment, 0);
   if (%client.player.getInventory(%itemType) > 0)
   {
      %client.needEquipment = false;
      %task.setMonitorFreq(15);
   }

   //then, buy the equipment or find it if you can :) - Lagg... 2-25-2004
   else if (%client.needEquipment)
   {
      // check to see if there's a deployable pack nearby
      %closestItem = -1;
      %closestItemDist = 32767;
      %itemType = getWord(%task.equipment, 0);

      //search the AIItemSet for a deployable pack (someone might have dropped one...)
      %armor = %client.player.getArmorSize();
      %itemCount = $AIItemSet.getCount();
      for (%i = 0; %i < %itemCount; %i++)
      {
         %item = $AIItemSet.getObject(%i);
         if (%item.getDataBlock().getName() $= %itemType && !%item.isHidden())
	 {
            %cant = false;
            if ((%itemType $= "InventoryDeployable" || %itemType $= "TurretOutdoorDeployable" || %itemType $= "TurretIndoorDeployable") && %armor $= "Light")
               %cant = true;

            %dist = %client.getPathDistance(%item.getWorldBoxCenter());
	    if (%dist > 0 && %dist < %closestItemDist && !%cant)
	    {
               %closestItem = %item;
	       %closestItemDist = %dist;
	    }
         }
      }

      //choose whether we're picking up the closest pack, or buying from an inv station...
      if ((isObject(%closestItem) && %closestItem != %task.pickupPack) || (%task.buyInvTime != %client.buyInvTime))
      {
         %task.pickupPack = %closestItem;

	 //initialize the inv buying
	 %task.buyInvTime = getSimTime();
	 AIBuyInventory(%client, %task.equipment, %task.buyEquipmentSet, %task.buyInvTime);

	 //now decide which is closer
	 if (isObject(%closestItem))
	 {
            //added a little here to check for power - Lagg... - 3-25-2003
	    if (isObject(%client.invToUse) && (%client.invToUse.isEnabled()) && (%client.invToUse.isPowered()))
	    {
               %dist = %client.getPathDistance(%client.invToUse.getWorldBoxCenter());
	       if (%dist < %closestItemDist)
	          %task.usingInv = true;
	       else
		  %task.usingInv = false;
            }
	    else
               %task.usingInv = false;
	 }
	 else
	    %task.usingInv = true;
      }

      //now see if we found a closer deployable pack
      if (!%task.usingInv && isObject(%task.pickupPack))
      {
         %client.stepMove(%task.pickupPack.position, 0.25);
         %distToPack = %client.getPathDistance(%task.pickupPack.position);
         if (%distToPack < 10 && %client.player.getMountedImage($BackpackSlot) > 0)
	    %client.player.throwPack();

	 //and we're finished until we actually have the new pack... we should - Lagg...
	 return;
      }
      else
      {
         %task.setMonitorFreq(5);
         %result = AIBuyInventory(%client, %task.equipment, %task.buyEquipmentSet, %task.buyInvTime);
	 if (%result $= "InProgress")
            return;
	 else if (%result $= "Finished")
	 {
	    %task.setMonitorFreq(15);
            %client.needEquipment = false;

            //if we made it past the inventory buying, reset the inv time
	    %task.buyInvTime = getSimTime();
         }
	 else if (%result $= "Failed")
	 {
            //if this task is the objective task, choose a new objective
	    if (%task == %client.objectiveTask)
            {
	       AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
	    }
	    return;
	 }
      }
   }

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25);
            return;
         }
      }
      else
         %task.useTeleport = false;
   }
   //--------------------------------------------------- monitor telporter - end -

   //make sure we still have equipment
		%client.needEquipment = AINeedEquipment(%task.equipment, %client);
   if (%client.needEquipment)
   {
      //if this task is the objective task, choose a new objective
      if (%task == %client.objectiveTask)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
	 return;
      }
   }

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
         if (%client.isAIControlled())
         {
            if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
	    }
         }
      }
   }

   //see if we're supposed to be engaging anyone...
   if (AIClientIsAlive(%client.shouldEngage))
   {
      %hasLOS = %client.hasLOSToClient(%client.shouldEngage);
      %losTime = %client.getClientLOSTime(%client.shouldEngage);
      if (%hasLOS || %losTime < 1000)
         %client.setEngageTarget(%client.shouldEngage);
      else
         %client.setEngageTarget(-1);
   }
   else
      %client.setEngageTarget(-1);

   //calculate the deployFromLocation
   %factor = -1 * (4 - (%task.passes * 0.5));//changed to 4m behind to improve proformance --- Lagg... 3-15-2003
   %task.deployFromLocation = VectorAdd(%task.location,VectorScale(%task.deployDirection, %factor));

   //see if we're within range of the deploy location
   %clLoc = %client.player.position;
   %distance = VectorDist(%clLoc, %task.deployFromLocation);
   %dist2D = VectorDist(%client.player.position, getWords(%task.deployFromLocation, 0, 1) SPC getWord(%client.player.position, 2));

   //set the aim when we get near the target...  this will be overwritten when we're actually trying to deploy
   if (%distance < 10 && %dist2D < 10)
      %client.aimAt(%task.location, 1000);

   if ((%client.pathDistRemaining(20) > %distance + 0.25) || %dist2D > 0.5)
   {
      %task.deployAttempts = 0;
      %task.checkObstructed = false;
      %task.waitMove = 0;
      %client.stepMove(%task.deployFromLocation, 0.25, $AIModeExpress);
      %task.setMonitorFreq(15);
      return;
   }
   
   if (%task.deployAttempts < 10 && %task.passes < 5 && !AIClientIsAlive(%client.getEngageTarget()))
   {
      //dissolve the human control link
      if (%task == %client.objectiveTask)
         aiReleaseHumanControl(%client.controlByHuman, %client);

      %task.setMonitorFreq(3);
      %client.stop();
      if (%task.deployAttempts == 0)
         %deployPoint = %task.location;
      else
	 %deployPoint = findTurretDeployPoint(%client, %task.location, %task.deployAttempts);
      if(%deployPoint !$= "")
      {
         // we have possible point
         %task.deployAttempts++;
         %client.aimAt(%deployPoint, 2000);

         //try to deploy the backpack
         %client.deployPack = true;
         %client.lastDeployedObject = -1;
         %client.player.use(Backpack);
         
         // check if pack deployed
         if (isObject(%client.lastDeployedObject))
         {
            //see if there's a "repairObject" objective for the newly deployed thingy...
            if (%task == %client.objectiveTask)
	    {
	       %deployedObject = %client.lastDeployedObject;

	       //search the current objective group and search for a "repair Object" task...
	       %objective = %client.objective;

	       //delete any previously associated "AIORepairObject" objective
	       if (isObject(%objective.repairObjective))
	       {
	          AIClearObjective(%objective.repairObjective);
		  %objective.repairObjective.delete();
		  %objective.repairObjective = "";
                  clearObjectiveFromTable(%objective.repairObjective);//new function below to stop objective spam - Lagg... 1-27-2004
	       }

	       //add the repair objective
	       %objective.repairObjective = new AIObjective(AIORepairObject)
	               {
	                       dataBlock = "AIObjectiveMarker";
		               weightLevel1 = %objective.weightLevel1 - 50;
		               weightLevel2 = 0;
		               description = "Repair the " @ %deployedObject.getDataBlock().getName();
                               mode = %task.mode;
			       targetObjectId = %deployedObject;
			       issuedByClientId = %client;
		               offense = false;
			       defense = true;
		               equipment = "RepairPack";
                               buyEquipmentSet = "MediumRepairSet";
		       };
                       %objective.repairObjective.deployed = true;
		       %objective.repairObjective.setTransform(%objective.getTransform());
		       %objective.repairObjective.group = %objective.group;
                       MissionCleanup.add(%objective.repairObjective);

               //if is siege game we have to do this right - Lagg... 11-3-2003
               if (Game.class $= "SiegeGame")
               {
                  //the objectives in $ObjectiveQ[1] are all offense objectives, $ObjectiveQ[2] has the defensive ones..
                  if (%client.team == game.offenseTeam)
                     $ObjectiveQ[1].add(%objective.repairObjective);
                  else
                     $ObjectiveQ[2].add(%objective.repairObjective);
               }
               else
                  $ObjectiveQ[%client.team].add(%objective.repairObjective);


               //changed to fix Siege gameType wrong Q error - Lagg... 11-5-2003
               //MissionCleanup.add(%objective.repairObjective);
               //$ObjectiveQ[%client.team].add(%objective.repairObjective);

               //finally, unassign the client so he'll go do something else...
               AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
               //finished
               return;
	    }
	 }
      }
   }
   else if (!%task.checkObstructed)
   {
      %task.checkObstructed = true;

      //see if anything is in our way
      InitContainerRadiusSearch(%task.location, 4, $TypeMasks::MoveableObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::PlayerObjectType);
      %objSrch = containerSearchNext();
      if (%objSrch == %client.player)
         %objSrch = containerSearchNext();
      if (%objSrch)
         AIMessageThread("ChatMove", %client, -1);
   }
   else if (%task.waitMove < 5 && %task.passes < 5)
   {
      %task.waitMove++;

      //try another pass at deploying 
      if (%task.waitMove == 5)
      {
         %task.waitMove = 0;
         %task.passes++;
         %task.deployAttempts = 0;

         //see if we're *right* underneath the deploy point
         %deployDist2D = VectorDist(getWords(%client.player.position, 0, 1) @ "0", getWords(%task.location, 0, 1) @ "0");
         if (%deployDist2D < 0.25)
	 {
	    %client.pressjump();
            %client.pressJet();
	    %client.deployPack = true;
	    %client.player.use(Backpack);

	    // check if pack deployed
	    if(%client.player.getMountedImage($BackpackSlot) == 0)
	    {
	       //don't add a "repairObject" objective for ceiling turrets
	       if (%task == %client.objectiveTask)
	       {
	          AIUnassignClient(%client);
		  Game.AIChooseGameObjective(%client);
	       }
	    }
	 }
      }
   }
   else
   {
      //find a new assignment - and remove this one from the Queue
      if (%task == %client.objectiveTask)
      {
         //error(getTaggedString(%client.name) SPC ", from team" SPC %client.team SPC ", messed up deploy objective:" SPC %client.objectiveTask.equipment SPC ", So we are resetting position");
                
         //%client.objective.isInvalid = true;
	 //AIUnassignClient(%client);
         //Game.AIChooseGameObjective(%client);
         %client.setDangerLocation(%task.location, 45);
         %task.passes = 0;
         %task.deployAttempts = 0;
         %task.checkObstructed = false;
         %task.waitMove = 0;
      }
   }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Laze Object +++
//fixed here so bots will acually use the targetting lazer for the right team :) - Lagg... - 4-12-2003
//modified last laze time and distance to client we laze for.
   
function AIOLazeObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //not if we are in the base
   if (%client.inBase)
      return 0;

   //lets check the range on this one
   if (vectorDist(%client.Player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter()) < 100)
      return 0;
      
	//see if it's already being lazed
	%numTargets = ServerTargetSet.getCount();
	for (%i = 0; %i < %numTargets; %i++)
	{
		%targ = ServerTargetSet.getObject(%i);
		if (%targ.sourceObject != %client.player)
		{
			%targDist = VectorDist(%targ.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter());
			if (%targDist < 10)
			{
				%this.lastLazedTime = getSimTime();
				%this.lastLazedClient = %targ.sourceObject.client;
				break;
 			}
		}
	}
   if (%this.issuedByClientId > 0 && %this.issuedByClinetId == %client)
   {
       echo("cant laze target for your self IDIOT !");//never see this message, so it must be ok - Lagg...
       return 0;
   }

   //no need to laze if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
	else if (%this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;      
	else if (getSimTime() - %this.lastLazedTime <= 10000 && %this.lastLazedClient == %client)
		return 0;
   else
	{
	   //set the base weight
	   switch (%level)
	   {
	      case 1:
	         %weight = %this.weightLevel1;
	      case 2:
	         %weight = %this.weightLevel2;
	      case 3:
	         %weight = %this.weightLevel3;
	      default:
	         %weight = %this.weightLevel4;
	   }
   
	   //check Affinity
	   if (ClientHasAffinity(%this, %client))
	      %weight += 100;
      
		//for now, do not deviate from the current assignment to laze a target, if you don't
		//already have a targeting laser.
		%needEquipment = AINeedEquipment(%this.equipment, %client);
		if (!%needEquipment)
			%weight += 100;
		else if (!aiHumanHasControl(%client.controlByHuman, %client))
			return 0;

		//see if this client is close to the issuing client
		if (%this.issuedByClientId > 0)
		{
			if (! AIClientIsAlive(%this.issuedByClientId))
				return 0;

		   %distance = %client.getPathDistance(%this.issuedByClientId.player.getWorldBoxCenter());
			if (%distance < 0)
				%distance = 32767;

		   //see if we're over 300 m
		   if (%distance > 300)
		      return 0;

                   //see if we're within 200 m
		   if (%distance < 200)
		      %weight += 30;

		   //see if we're within 90 m
		   if (%distance < 90)
		      %weight += 30;

		   //see if we're within 45 m
		   if (%distance < 45)
		      %weight += 30;
		}

		//now, if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman && %weight < $AIWeightHumanIssuedCommand)
			%weight = $AIWeightHumanIssuedCommand;

		return %weight;
	}
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Missile Object +++
//                                                                            - Lagg... 4-12-2003

function AIOMissileObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;      
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //check the zone
   if (%client.inBase)
      return 0;

   //lets check the range on this one
   if (vectorDist(%client.Player.getWorldBoxCenter(), %this.getWorldBoxCenter()) < 50)
      return 0;
    
   //no need to attack if the object is already destroyed
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
	{
		//if this bot is linked to a human who has issued this command, up the weight
		if (%this.issuedByClientId == %client.controlByHuman)
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
			{
				if ($AIWeightHumanIssuedCommand < %minWeight)
					return 0;
				else
					%weight = $AIWeightHumanIssuedCommand;
			}
			else
			{
				// calculate the default...
				%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
				if (%weight < $AIWeightHumanIssuedCommand)
					%weight = $AIWeightHumanIssuedCommand;
			}
		}
		else
		{
			//make sure we have the potential to reach the minWeight
			if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
				return 0;

			// calculate the default...
			%weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
		}

		return %weight;
	}
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ AIO Mortar Object +++
//Modified so bot gets LOS to target before firing. Fixed a lot of stuff here.          - Lagg...

function AIOMortarObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   // if were playing CnH, check who owns this
   if (!isObject(%this.targetObjectId) || %this.targetObjectId.isHidden() || %this.targetObjectId.team == %client.team)
      return 0;
   
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //check the zone
   if (%client.inBase)
      return 0;

   //lets check the range on this one
   if (vectorDist(%client.Player.getWorldBoxCenter(), %this.targetObjectId.getWorldBoxCenter()) < 100)
      return 0;

   //check equipment requirement
	%needEquipment = AINeedEquipment(%this.equipment, %client);
        %needArmor = AIMustUseRegularInvStation(%this.equipment, %client);
        %result = AIFindClosestInventory(%client, %needArmor);
        %closestInv = getWord(%result, 0);
        %closestDist = getWord(%result, 1);
 

        //if (%client.team == Game.offenseTeam && %closestDist > 100 && (%needArmor || (%this.equipment $= "DeployedStationInventory" && %needEquipment)))
        if (%client.team == Game.offenseTeam && %closestDist > 100 && (%needArmor || %needEquipment))
           return 0;
      
   //no need to attack if the object is already destroyed
   if (%this.targetObjectId.getDamageState() $= "Destroyed")
      return 0;
   else
   {
      //if this bot is linked to a human who has issued this command, up the weight
      if (%this.issuedByClientId == %client.controlByHuman)
      {
         //make sure we have the potential to reach the minWeight
	 if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	 {
	    if ($AIWeightHumanIssuedCommand < %minWeight)
	       return 0;
	    else
	       %weight = $AIWeightHumanIssuedCommand;
	 }
	 else
	 {
	    // calculate the default...
	    %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
	    if (%weight < $AIWeightHumanIssuedCommand)
	       %weight = $AIWeightHumanIssuedCommand;
         }
      }
      else
      {
         //make sure we have the potential to reach the minWeight
         if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
	    return 0;

	 // calculate the default...
         %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);
      }

      return %weight;
   }
}

//--------------------------------------------------------------------------------------------------- AIO Touch Object ---

function AIOTouchObject::weight(%this, %client, %level, %minWeight, %inventoryStr)
{
   //make sure the player is still alive!!!!!
   if (! AIClientIsAlive(%client))
      return 0;

   //special case for Gen Room Camping
   if (isObject(%client.objectiveTask))
   {
      %wat = %client.objectiveTask.getName();
      if (%wat  $= "AIDefendLocation" || %wat $= AIDeployEquipment)
         return 0;
   }

   if (!AIODefault::QuickWeight(%this, %client, %level, %minWeight))
      return 0;

   switch$ (%this.mode)
   {
      case "TouchFlipFlop":
         if(%this.targetObjectId.team == %client.team || %this.targetObjectId.isHidden())
	    return 0;        

         //added a check for siege games to enhance the ai, fix navGraph bug with forcefileds
         //check if ff is up around switch, if so don't do task - Lagg...
         else if ($CurrentMissionType $= "Siege")
         {
            //Special case to account for force fields up or down and if we are in the base or not
            if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
               return 0;

            else if (nameToId(ObjectiveGroup).powerCount > 0)
               return 0;

            else if (nameToId(MainPower).powerCount > 0)
               return 0;
         }

	 %weight = AIODefault::weight(%this, %client, %level, %inventoryStr);

         if (%client.inBase)
            %weight += 200;

         return %weight;

      case "FlagGrab":
			if (! %this.targetObjectId.isHome)
				return 0;
			else
	         return AIODefault::weight(%this, %client, %level, %inventoryStr);
      case "FlagDropped":
			if ((%this.targetObjectId.isHome) || (%this.targetObjectId.carrier !$= ""))
				return 0;
			else
	         return AIODefault::weight(%this, %client, %level, %inventoryStr);
      case "FlagCapture":
			if (%this.targetObjectId.carrier != %client.player)
            return 0;
         else
			{
				//find our home flag location
				%homeTeam = %client.team;
				%homeFlag = $AITeamFlag[%homeTeam];
				%this.location = %homeFlag.originalPosition;
				return AIODefault::weight(%this, %client, %level, %inventoryStr);
			}
   }
   return 0;
}

//----------------------------------------------------------------------------------------------------- AI Touch Object ---

function AITouchObject::monitor(%task, %client)
{
   if ($CurrentMissionType $= "Siege")
   {
      //Special case to account for force fields up or down and if we are in the base or not
      if (nameToId("EntranceFFGroup").powerCount > 0 && !%client.inBase)
      {
         if (%task == %client.objectivetask)
         {
            AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
               return;
         }
      }
      else if (nameToId(ObjectiveGroup).powerCount > 0)
      {
         if (%task == %client.objectivetask)
         {
            AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
               return;
         }
      }
      else if (nameToId(MainPower).powerCount > 0)
      {
         if (%task == %client.objectivetask)
         {
            AIUnassignClient(%client);
	       Game.AIChooseGameObjective(%client);
               return;
         }
      }
   }

   //--------------------------------------------------- monitor telporter - start -
   //are we using teleporter
   if (%task.useTeleport)
   {
      %result = AIFindDeployedMPB(%client);//new function in aiVehicle.cs
      %closestMPB = getWord(%result, 0);
      %closestMPBDist = getWord(%result, 1);
      %result = SweepForTeleporters(%client);//new function in aiVehicle.cs
      %closestTel = getWord(%result, 0);
      %closestTelDist = getWord(%result, 1);
      if (%closestMPB > 0 && %closestTel > 0)
      {
         //we are done teleporting
         if (%closestMPBDist < %closestTelDist)
         {
            //reset the clients inventory status
            if (%client.needEquipment)
            {
               %result = AIFindClosestInventory(%client, 0);
	       %closestInv = getWord(%result, 0);
               %client.invToUse = %closestInv;
            }
            %task.useTeleport = false;
            return;
         }

         if (%closestTelDist < 2)
            %client.pressJump();
         else
         {
            %client.stepMove(%closestTel.getWorldBoxCenter(), 0.25);
            return;
         }
      }
      else
         %task.useTeleport = false;
   }
   //--------------------------------------------------- monitor telporter - end -

   if (%client.inBase)
      %client.needEquipment = false;

   // ---------------------- buy equipment
   if (%client.needEquipment)
   {
      %task.setMonitorFreq(5);
      if (%task.equipment !$= "")
         %equipmentList = %task.equipment;
      else
	 %equipmentList = %task.desiredEquipment;
      %result = AIBuyInventory(%client, %equipmentList, %task.buyEquipmentSet, %task.buyInvTime);
      if (%result $= "InProgress")
         return;
      else if (%result $= "Finished")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
      else if (%result $= "Failed")
      {
         %task.setMonitorFreq(15);
	 %client.needEquipment = false;
      }
   }

   //if we made it past the inventory buying, reset the inv time
   %task.buyInvTime = getSimTime();

   //chat
   if (%task.sendMsg)
   {
      if (%task.sendMsgTime == 0)
         %task.sendMsgTime = getSimTime();
      else if (getSimTime() - %task.sendMsgTime > 7000)
      {
         %task.sendMsg = false;
	 if (%client.isAIControlled())
	 {
	    if (%task.chat !$= "")
	    {
	       %chatMsg = getWord(%task.chat, 0);
	       %chatTemplate = getWord(%task.chat, 1);
	       if (%chatTemplate !$= "")
	          AIMessageThreadTemplate(%chatTemplate, %chatMsg, %client, -1);
	       else
		  AIMessageThread(%task.chat, %client, -1);
            }
         }
      }
   }

   //keep updating the position, in case the flag is flying through the air...
   if (%task.location !$= "")
      %touchPos = %task.location;
   else
      %touchPos = %task.targetObject.getWorldBoxCenter();
      
   //see if we need to engage a new target
   %engageTarget = %client.getEngageTarget();
   if (!AIClientIsAlive(%engageTarget) && %task.engageTarget > 0)
      %client.setEngageTarget(%task.engageTarget);
   
   //else see if we should abandon the engagement
   else if (AIClientIsAlive(%engageTarget))
   {
      %myPos = %client.player.getWorldBoxCenter();
      %testPos = %engageTarget.player.getWorldBoxCenter();
      %distance = %client.getPathDistance(%testPos);
      if (%distance < 0 || %distance > 50)
         %client.setEngageTarget(-1);
   }

   //see if we have completed our objective
   if (%task == %client.objectiveTask)
   {
      %completed = false;
      switch$ (%task.mode)
      {
         case "TouchFlipFlop":
	    if (%task.targetObject.team == %client.team)
	       %completed = true;
	 case "FlagGrab":
	    if (!%task.targetObject.isHome)
               %completed = true;
	 case "FlagDropped":
	    if ((%task.targetObject.isHome) || (%task.targetObject.carrier !$= ""))
	       %completed = true;
	 case "FlagCapture":
	    if (%task.targetObject.carrier != %client.player)
	       %completed = true;
      }
      if (%completed)
      {
         AIUnassignClient(%client);
	 Game.AIChooseGameObjective(%client);
	 return;
      }
   }

   if (%task.mode $= "FlagCapture")
   {
      %homeFlag = $AITeamFlag[%client.team];

      //if we're within range of the flag's home position, and the flag isn't home, start idling...
      if (VectorDist(%client.player.position, %touchPos) < 40 && !%homeFlag.isHome)
      {
         if (%client.getStepName() !$= "AIStepIdlePatrol")
	    %client.stepIdle(%touchPos);
      }
      else
         %client.stepMove(%touchPos, 0.25, $AIModeExpress);
   }

   //if siege game we want a special way of capturing the flip flop - Lagg... 9-11-2003
   else if ((%task.mode $= "TouchFlipFlop") && ($CurrentMissionType $= "Siege"))
   {

      //if there is a force field blocking the way - Lagg... 9-11-2003
      InitContainerRadiusSearch(%touchPos, 14, $TypeMasks::ForceFieldObjectType);
      %objSearch = containerSearchNext();
      if (%objSearch > 0 && %objSearch.isPowered())//should we check for all pass force fields? - Lagg ... (nah)
      {
         if (%task == %client.objectiveTask)
         {
            AIUnassignClient(%client);
	    Game.AIChooseGameObjective(%client);
	    return;
         }
      }
      else
         %client.stepMove(%touchPos, 0.25, $AIModeExpress);
   }
   else
      %client.stepMove(%touchPos, 0.25, $AIModeExpress);

   if (VectorDist(%client.player.position, %touchPos) < 10)
   {
      //dissolve the human control link
      if (%task == %client.objectiveTask)
      {
         if (aiHumanHasControl(%task.issuedByClient, %client))
	 {
	    aiReleaseHumanControl(%client.controlByHuman, %client);

	    //should re-evaluate the current objective weight
	    %inventoryStr = AIFindClosestInventories(%client);
	    %client.objectiveWeight = %client.objective.weight(%client, %client.objectiveLevel, 0, %inventoryStr);
	 }
      }
   }

   //see if we're supposed to be engaging anyone...
   if (!AIClientIsAlive(%client.getEngageTarget()) && AIClientIsAlive(%client.shouldEngage))
      %client.setEngageTarget(%client.shouldEngage);
}

//--------------------------------------------------------------------------------------------------------------
function AIConnection::onAIConnect(%client, %name, %team, %skill, %offense, %voice, %voicePitch)
{
   // Sex/Race defaults
   %client.sex = "Male";
   %client.race = "Human";
   %client.armor = "Light";

   //setup the voice and voicePitch
   if (%voice $= "")
      %voice = "Bot1";
   %client.voice = %voice;
   %client.voiceTag = addTaggedString(%voice);
   
   if (%voicePitch $= "" || %voicePitch < 0.5 || %voicePitch > 2.0)
      %voicePitch = 1.0;
	%client.voicePitch = %voicePitch;

   %client.name = addTaggedString( "\cp\c9" @ %name @ "\co" );
	%client.nameBase = %name;

   echo(%client.name);
   echo("CADD: " @ %client @ " " @ %client.getAddress());
   $HostGamePlayerCount++;
   
   //set the initial team - Game.assignClientTeam() should be called later on...
   %client.team = %team;
   if ( %client.team & 1 )
      %client.skin = addTaggedString( "basebot" );
   else
      %client.skin = addTaggedString( "basebbot" );

	//setup the target for use with the sensor net, etc...
   %client.target = allocClientTarget(%client, %client.name, %client.skin, %client.voiceTag, '_ClientConnection', 0, 0, %client.voicePitch);
   
   //i need to send a "silent" version of this for single player but still use the callback  -jr`
   if($currentMissionType $= "SinglePlayer")
		messageAllExcept(%client, -1, 'MsgClientJoin', "", %name, %client, %client.target, true);
   else
	   messageAllExcept(%client, -1, 'MsgClientJoin', '\c1%1 joined the game.', %name, %client, %client.target, true);

	//assign the skill
	%client.setSkillLevel(%skill);
	
	//assign the affinity
   %client.offense = %offense;
    
   //clear any flags
   %client.stop(); // this will clear the players move state
   %client.clearStep();
   %client.lastDamageClient = -1;
   %client.lastDamageTurret = -1;
   %client.setEngageTarget(-1);
   %client.setTargetObject(-1);
   %client.engageRemeq = -1;//added - Lagg...
   %client.pickUpItem = -1;//added - Lagg...
   %client.objective = "";

   //reset the client zone flags for mission No Quarter  - Lagg... 12-17-2003
   %client.inBase = false;
   //%client.inRoom = false;

	//clear the defaulttasks flag
	%client.defaultTasksAdded = false;

	//if the mission is already running, spawn the bot
   if ($missionRunning)
      %client.startMission();
}

//-----------------------------------------------------------------------------------------------------------------------
function onAIRespawn(%client)
{
   %markerObj = Game.pickPlayerSpawn(%client, true);
   Game.createPlayer(%client, %markerObj);	

	//make sure the player object is the AI's control object - even during the mission warmup time
	//the function AISystemEnabled(true/false) will control whether they actually move...
	%client.setControlObject(%client.player);
   
   if (%client.objective)
      error("ERROR!!! " @ %client @ " is still assigned to objective: " @ %client.objective);
      
   //clear the objective and choose a new one
	AIUnassignClient(%client);
   %client.stop();
   %client.clearStep();
   %client.lastDamageClient = -1;
   %client.lastDamageTurret = -1;
   %client.shouldEngage = -1;
   %client.setEngageTarget(-1);
   %client.setTargetObject(-1);
   %client.engageRemeq = -1;//added - Lagg...
   %client.pickUpItem = -1;//added - Lagg...
	%client.pilotVehicle = false;

   //reset the client zone flags for mission No Quarter  - Lagg... 12-17-2003
   %client.inBase = false;
   //%client.inRoom = false;

	//set the spawn time
	%client.spawnTime = getSimTime();
	%client.respawnThread = "";

	//timeslice the objective reassessment for the bots
	if (!isEventPending(%client.objectiveThread))
	{
		%curTime = getSimTime();
		%remainder = %curTime % 5000;
		%schedTime = $AITimeSliceReassess - %remainder;
		if (%schedTime <= 0)
			%schedTime += 5000;
		%client.objectiveThread = schedule(%schedTime, %client, "AIReassessObjective", %client);

		//set the next time slice "slot"
		$AITimeSliceReassess += 300;
		if ($AITimeSliceReassess > 5000)
			$AITimeSliceReassess -= 5000;
	}

	//call the game specific spawn function
	Game.onAIRespawn(%client);
}

// ----------------------------------------------------------------------------------------------------------------------
function DefaultGame::AIChangeTeam(%game, %client, %newTeam)
{
   //make sure we're trying to drop an AI
   if (!isObject(%client) || !%client.isAIControlled())
      return;

   //clear the ai from any objectives, etc...
	AIUnassignClient(%client);
   %client.stop();
	%client.clearTasks();
   %client.clearStep();
   %client.lastDamageClient = -1;
   %client.lastDamageTurret = -1;
   %client.shouldEngage = -1;
   %client.setEngageTarget(-1);
   %client.setTargetObject(-1);
	%client.pilotVehicle = false;
   %client.defaultTasksAdded = false;

   //kill the player, which should cause the Game object to perform whatever cleanup is required.
   if (isObject(%client.player))
      %client.player.scriptKill(0);

   //clean up the team rank array
   %game.removeFromTeamRankArray(%client);

   //assign the new team
   %client.team = %newTeam;
   if (%newTeam < 0)
      Game.assignClientTeam(%client);
   else
   {
      if ( %client.team & 1 )
      {
         %client.skin = addTaggedString( "basebot" );
         setTargetSkin( %client.target, 'basebot' );
      }
      else
      {
         %client.skin = addTaggedString( "basebbot" );
         setTargetSkin( %client.target, 'basebbot' );
      }
   }
   
   messageAllExcept( %client, -1, 'MsgClientJoinTeam', '\c1bot %1 has switched to team %2.', %client.name, %game.getTeamName(%client.team), %client, %client.team );
}

//-----------------------------------------------------------------------------------------------------------------------
function AIMissionEnd()
{
	//disable the AI system
	AISystemEnabled(false);

	//loop through the client list, and clear the tasks of each bot
	%count = ClientGroup.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%client = ClientGroup.getObject(%i);
		if (%client.isAIControlled())
		{
			//cancel the respawn thread and the objective thread...
			cancel(%client.respawnThread);
			cancel(%client.objectiveThread);

         //reset the clients tasks, variables, etc...
	      AIUnassignClient(%client);
         %client.stop();
			%client.clearTasks();
         %client.clearStep();
         %client.lastDamageClient = -1;
         %client.lastDamageTurret = -1;
         %client.shouldEngage = -1;
         %client.setEngageTarget(-1);
         %client.setTargetObject(-1);
         %client.engageRemeq = -1;//added - Lagg...
         %client.pickUpItem = -1;//added - Lagg...
	      %client.pilotVehicle = false;
         %client.defaultTasksAdded = false;

         //reset the client zone flags - Lagg... 12-17-2003
         %client.inBase = false;
         //%client.inRoom = false;

	      //do the nav graph cleanup
	      %client.missionCycleCleanup();
		}
	}

	//clear the objective Q's
	for (%i = 0; %i <= Game.numTeams; %i++)
	{
		if (isObject($ObjectiveQ[%i]))
		{
			$ObjectiveQ[%i].clear();
			$ObjectiveQ[%i].delete();
		}
		$ObjectiveQ[%i] = "";
	}

	//now delete all the sets used by the AI system...
	if (isObject($AIBombLocationSet))
   	$AIBombLocationSet.delete();
	$AIBombLocationSet = "";

	if (isObject($AIInvStationSet))
   	$AIInvStationSet.delete();
	$AIInvStationSet = "";

	if (isObject($AIItemSet))
   	$AIItemSet.delete();
	$AIItemSet = "";

	if (isObject($AIGrenadeSet))
   	$AIGrenadeSet.delete();
	$AIGrenadeSet = "";

	if (isObject($AIWeaponSet))
   	$AIWeaponSet.delete();
	$AIWeaponSet = "";

	if (isObject($AIRemoteTurretSet))
   	$AIRemoteTurretSet.delete();
	$AIRemoteTurretSet = "";

	if (isObject($AIDeployedMineSet))
   	$AIDeployedMineSet.delete();
	$AIDeployedMineSet = "";

	if (isObject($AIVehicleSet))
   	$AIVehicleSet.delete();
	$AIVehicleSet = "";

        //get rid of the aivehiclepadset as well ----------------------------------------- added Lagg... - 8-19-2003
        if (isObject($AIVehiclePadSet))
   	$AIVehiclePadSet.delete();
	$AIVehiclePadSet = "";

        //get rid of the aiteleportset as well ----------------------------------------- added Lagg... - 10-1-2003
        if (isObject($AITeleportSet))
   	$AITeleportSet.delete();
	$AITeleportSet = "";
}

// --------------------------------------------------------------------------------------------------------------------------------------
